#include <Python.h>
#include <pnet/il_engine.h>
#include <pnet/il_image.h>

typedef struct 
{
    PyObject_HEAD;
	ILExecProcess *process;
    /* Type-specific fields go here. */
} pnet_ILExecProcessObject;

static void pnet_ILExecProcess_dealloc(PyObject* self)
{
	ILExecProcessDestroy(((pnet_ILExecProcessObject*)self)->process);
}

static PyTypeObject pnet_ILExecProcessType = 
{
    PyObject_HEAD_INIT(NULL)
    0,
    "ILExecProcess",
    sizeof(pnet_ILExecProcessObject),
    0,
    pnet_ILExecProcess_dealloc, /*tp_dealloc*/
    0,          /*tp_print*/
    0,          /*tp_getattr*/
    0,          /*tp_setattr*/
    0,          /*tp_compare*/
    0,          /*tp_repr*/
    0,          /*tp_as_number*/
    0,          /*tp_as_sequence*/
    0,          /*tp_as_mapping*/
    0,          /*tp_hash */
};


static PyObject* pnet_ILInitLocale(PyObject *self,PyObject *args)
{
	ILInitLocale();
	return PyInt_FromLong(1);
}

static PyObject* pnet_ILExecInit(PyObject *self,PyObject *args)
{
	int arg0=0;
	PyArg_ParseTuple(args,"i:ILExecInit",&arg0);
	ILExecInit(arg0);
	return PyInt_FromLong(1);
}

/* 'self' is not used */
static PyObject *
pnet_ILExecProcessCreate(PyObject *self, PyObject* args)
{
	ILExecProcess *process;
	pnet_ILExecProcessObject *obj;
	long arg0=0;
	PyArg_ParseTuple(args,"i:ILProcessCreate",&arg0);
	process = ILExecProcessCreate(arg0);
	if(!process)
		return PyInt_FromLong(0); //false
	obj = PyObject_NEW (pnet_ILExecProcessObject,&pnet_ILExecProcessType);
	obj->process=process;
	return (PyObject*)obj;
}

static PyObject *
pnet_ILExecProcessSetLibraryDirs(PyObject *self, PyObject *args)
{
	pnet_ILExecProcessObject* obj;
	PyListObject *list;
	int count;
	int i=0;
	char **data=NULL;
	PyArg_ParseTuple(args,"OOi:ILExecProcessSetLibraryDirs",&obj,&list,&count);
	if(count > PyList_Size((PyObject*)list))
	{
		return PyInt_FromLong(0);
	}
	data=(char**)calloc(sizeof(char*),count);
	for(i=0;i<count;i++)
	{
		if(PyString_Check(PyList_GetItem((PyObject*)list,i)))
		{
			return PyInt_FromLong(0);
		}
		data[i]=PyString_AsString(PyList_GetItem((PyObject*)list,i));
	}
	ILExecProcessSetLibraryDirs(obj->process,data,count);
	free(data);
	return PyInt_FromLong(1);
}

static PyObject*
pnet_ILExecProcessLoadFile(PyObject* self,PyObject *args)
{
	pnet_ILExecProcessObject* obj;
	PyObject *str;
	PyArg_ParseTuple(args,"OS:ILExecProcessLoadFile",&obj,&str);
	return PyInt_FromLong(\
			ILExecProcessLoadFile(obj->process,PyString_AsString(str))+1);
}

typedef struct 
{
    PyObject_HEAD;
	ILExecThread *thread;
    /* Type-specific fields go here. */
} pnet_ILExecThreadObject;

static void pnet_ILExecThread_dealloc(PyObject* self)
{
	ILExecThreadDestroy(((pnet_ILExecThreadObject*)self)->thread);
}

static PyTypeObject pnet_ILExecThreadType = 
{
    PyObject_HEAD_INIT(NULL)
    0,
    "ILExecThread",
    sizeof(pnet_ILExecThreadObject),
    0,
    pnet_ILExecThread_dealloc, /*tp_dealloc*/
    0,          /*tp_print*/
    0,          /*tp_getattr*/
    0,          /*tp_setattr*/
    0,          /*tp_compare*/
    0,          /*tp_repr*/
    0,          /*tp_as_number*/
    0,          /*tp_as_sequence*/
    0,          /*tp_as_mapping*/
    0,          /*tp_hash */
};

static PyObject *
pnet_ILExecProcessGetMain(PyObject *self,PyObject *args)
{
	pnet_ILExecProcessObject* obj;
	pnet_ILExecThreadObject* thread;
	PyArg_ParseTuple(args,"O:ILExecProcessGetMain",&obj);
	if(!obj)
		return PyInt_FromLong(0);
	thread = PyObject_NEW(pnet_ILExecThreadObject,&pnet_ILExecThreadType);
	thread->thread=ILExecProcessGetMain(obj->process);
	if(!thread->thread)
		return PyInt_FromLong(0);		
	return (PyObject*)thread;
}

typedef struct 
{
    PyObject_HEAD;
	ILObject *object;
	ILExecThread *thread;
    /* Type-specific fields go here. */
} pnet_ILObject;

static void pnet_ILObject_dealloc(PyObject* self)
{
	//TODO
}

static PyObject* 
pnet_ILObject_ToString(PyObject *self)
{
	ILString *val=0;
	
	if(((pnet_ILObject*)self)->object)
	{
		/* Call the virtual "ToString" method on the object */
		ILExecThreadCallNamedVirtual(((pnet_ILObject*)self)->thread, 
									"System.Object",
									"ToString",
									"(T)oSystem.String;",
									&val, 
									((pnet_ILObject*)self)->object);
	}
	return PyString_FromString(ILStringToAnsi(((pnet_ILObject*)self)->thread,val));
}

static PyTypeObject pnet_ILObjectType = 
{
    PyObject_HEAD_INIT(NULL)
    0,
    "ILObject",
    sizeof(pnet_ILObject),
    0,
    pnet_ILObject_dealloc, /*tp_dealloc*/
    0,          /*tp_print*/
    0,          /*tp_getattr*/
    0,          /*tp_setattr*/
    0,          /*tp_compare*/
    pnet_ILObject_ToString,          /*tp_repr*/
    0,          /*tp_as_number*/
    0,          /*tp_as_sequence*/
    0,          /*tp_as_mapping*/
    0,          /*tp_hash */
};

static PyObject *
pnet_ILExecThreadNew(PyObject *self,PyObject *args)
{
	pnet_ILExecThreadObject* thread;
	pnet_ILObject *obj;
	PyObject *name,*sig;
	char *format;
	char param1;
	int param2;
	thread=(pnet_ILExecThreadObject*)PyTuple_GetItem(args,0);
	name=PyTuple_GetItem(args,1);
	sig=PyTuple_GetItem(args,2);
	format=PyString_AsString(sig);
	if(strcmp("(Tci)V",format))
	{
		printf("Oops ! I forgot to implement VA_ARG stuff !\n");
		return PyInt_FromLong(0);		
	}
	param1=(PyString_AsString(PyTuple_GetItem(args,3)))[0];//char
	param2=(PyInt_AsLong(PyTuple_GetItem(args,4)));//int
	obj=PyObject_NEW(pnet_ILObject,&(pnet_ILObjectType));
	obj->object=ILExecThreadNew(thread->thread,PyString_AsString(name),
					format,param1,param2);//hopefully it will work !
	obj->thread=thread->thread;
	if(!obj->object)
			return PyInt_FromLong(0);
	return (PyObject*)obj;
}

static PyMethodDef pnet_methods[] = {
	{"ILExecProcessCreate",		pnet_ILExecProcessCreate,	METH_VARARGS,
	 "ILExecProcessCreate"},
	{"ILInitLocale",		pnet_ILInitLocale,	METH_VARARGS,
	 "ILInitLocale"},
	{"ILExecInit",		pnet_ILExecInit,	METH_VARARGS,
	 "ILExecInit"},
	{"ILExecProcessSetLibraryDirs",	pnet_ILExecProcessSetLibraryDirs,
			METH_VARARGS, "ILExecProcessSetLibraryDirs"},
	{"ILExecProcessLoadFile",		pnet_ILExecProcessLoadFile,	METH_VARARGS,
	 "ILExecProcessLoadFile"},
	 {"ILExecProcessGetMain", pnet_ILExecProcessGetMain, METH_VARARGS,
		"ILExecProcessGetMain"},
	 {"ILExecThreadNew", pnet_ILExecThreadNew, METH_VARARGS,
		"ILExecThreadNew"},
	{NULL,		NULL}		/* sentinel */
};

void
initpnet(void)
{
	PyImport_AddModule("pnet");
	Py_InitModule("pnet", pnet_methods);
}
